@@ -130,6 +130,7 @@ int loadParams(cgiRequestObj *request,
   if(getenv2("REQUEST_METHOD", thread_context)==NULL) {
     msIO_printf("This script can only be used to decode form results and \n");
     msIO_printf("should be initiated as a CGI process via a httpd server.\n");
+    msIO_printf("For other options please try using the --help switch.\n");
     return -1;
   }
 
@@ -386,33 +387,15 @@ int rind(char *s, char c)
   return -1;
 }
 
-int _getline(char *s, int n, FILE *f)
-{
-  register int i=0;
-
-  while(1) {
-    s[i] = (char)fgetc(f);
-
-    if(s[i] == CR)
-      s[i] = fgetc(f);
-
-    if((s[i] == 0x4) || (s[i] == LF) || (i == (n-1))) {
-      s[i] = '\0';
-      return (feof(f) ? 1 : 0);
-    }
-    ++i;
-  }
-}
-
 void send_fd(FILE *f, FILE *fd)
 {
-  char c;
+  int c;
 
   while (1) {
     c = fgetc(f);
-    if(feof(f))
+    if(c == EOF)
       return;
-    fputc(c,fd);
+    fputc((char)c,fd);
   }
 }
 
