@@ -1001,6 +1001,7 @@ int RebuildClusters(layerObj *layer, int isQuery)
 #ifdef USE_CLUSTER_EXTERNAL
   int layerIndex;
 #endif
+  reprojectionObj* reprojector = NULL;
 
   msClusterLayerInfo* layerinfo = layer->layerinfo;
 
@@ -1070,10 +1071,8 @@ int RebuildClusters(layerObj *layer, int isQuery)
   layerinfo->searchRect = searchrect;
 
   /* reproject the rectangle to layer coordinates */
-#ifdef USE_PROJ
   if((map->projection.numargs > 0) && (layer->projection.numargs > 0))
     msProjectRect(&map->projection, &layer->projection, &searchrect); /* project the searchrect to source coords */
-#endif
 
   /* determine the compare method */
   layerinfo->fnCompare = CompareRectangleRegion;
@@ -1133,11 +1132,18 @@ int RebuildClusters(layerObj *layer, int isQuery)
   if ((current = clusterInfoCreate(layerinfo)) == NULL)
     return MS_FAILURE;
 
-  while((status = msLayerNextShape(srcLayer, &current->shape)) == MS_SUCCESS) {
-#if defined(USE_PROJ) && defined(USE_CLUSTER_EXTERNAL)
-    /* transform the shape to the projection of this layer */
+#if defined(USE_CLUSTER_EXTERNAL)
     if(srcLayer->transform == MS_TRUE && srcLayer->project && layer->transform == MS_TRUE && layer->project &&msProjectionsDiffer(&(srcLayer->projection), &(layer->projection)))
-      msProjectShape(&srcLayer->projection, &layer->projection, &current->shape);
+    {
+        reprojector = msProjectCreateReprojector(&srcLayer->projection, &layer->projection);
+    }
+#endif
+  
+  while((status = msLayerNextShape(srcLayer, &current->shape)) == MS_SUCCESS) {
+#if defined(USE_CLUSTER_EXTERNAL)
+    /* transform the shape to the projection of this layer */
+    if( reprojector )
+      msProjectShapeEx(reprojector, &current->shape);
 #endif
     /* set up positions and variance */
     current->avgx = current->x = current->shape.bounds.minx;
@@ -1173,6 +1179,7 @@ int RebuildClusters(layerObj *layer, int isQuery)
       /* add this shape to the tree */
       if (treeNodeAddShape(layerinfo, layerinfo->root, current, depth) != MS_SUCCESS) {
         clusterInfoDestroyList(layerinfo, current);
+        msProjectDestroyReprojector(reprojector);
         return MS_FAILURE;
       }
     }
@@ -1190,6 +1197,7 @@ int RebuildClusters(layerObj *layer, int isQuery)
         /* if not found add this shape as a new cluster */
         if (treeNodeAddShape(layerinfo, layerinfo->root, current, depth) != MS_SUCCESS) {
           clusterInfoDestroyList(layerinfo, current);
+          msProjectDestroyReprojector(reprojector);
           return MS_FAILURE;
         }
       }
@@ -1197,10 +1205,13 @@ int RebuildClusters(layerObj *layer, int isQuery)
 
     if ((current = clusterInfoCreate(layerinfo)) == NULL) {
       clusterInfoDestroyList(layerinfo, current);
+      msProjectDestroyReprojector(reprojector);
       return MS_FAILURE;
     }
   }
 
+  msProjectDestroyReprojector(reprojector);
+
   clusterInfoDestroyList(layerinfo, current);
 
   if (layerinfo->algorithm == MSCLUSTER_ALGORITHM_FULL) {
