@@ -187,18 +187,17 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
    * These lines will be used when generating labels to get correct placement at arc/rect edge intersections.
    */
   rectMapCoordinates = layer->map->extent;
-#ifdef USE_PROJ
+
   layer->project = msProjectionsDiffer(&(layer->projection), &(layer->map->projection));
   if( layer->project &&
       strstr(layer->map->projection.args[0], "epsg:3857") &&
-      pj_is_latlong(layer->projection.proj) )
+      msProjIsGeographicCRS(&(layer->projection)) )
   {
       if( rectMapCoordinates.minx < -20037508)
           rectMapCoordinates.minx = -20037508;
       if( rectMapCoordinates.maxx > 20037508 )
           rectMapCoordinates.maxx = 20037508;
   }
-#endif
 
   msFree(pInfo->pboundinglines);
   pInfo->pboundinglines   = (lineObj *)  msSmallMalloc( sizeof( lineObj )  * 4 );
@@ -217,10 +216,8 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
     pInfo->pboundinglines[0].point[1].x = rectMapCoordinates.maxx;
     pInfo->pboundinglines[0].point[1].y = rectMapCoordinates.maxy;
 
-#ifdef USE_PROJ
     if(layer->project)
       msProjectLine(&layer->map->projection, &layer->projection, &pInfo->pboundinglines[0]);
-#endif
 
     /*
      * bottom
@@ -232,10 +229,8 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
     pInfo->pboundinglines[1].point[1].x   = rectMapCoordinates.maxx;
     pInfo->pboundinglines[1].point[1].y = rectMapCoordinates.miny;
 
-#ifdef USE_PROJ
     if(layer->project)
       msProjectLine(&layer->map->projection, &layer->projection, &pInfo->pboundinglines[1]);
-#endif
 
     /*
      * left
@@ -247,10 +242,8 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
     pInfo->pboundinglines[2].point[1].x   = rectMapCoordinates.minx;
     pInfo->pboundinglines[2].point[1].y   = rectMapCoordinates.maxy;
 
-#ifdef USE_PROJ
     if(layer->project)
       msProjectLine(&layer->map->projection, &layer->projection, &pInfo->pboundinglines[2]);
-#endif
 
     /*
      * right
@@ -262,10 +255,8 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
     pInfo->pboundinglines[3].point[1].x = rectMapCoordinates.maxx;
     pInfo->pboundinglines[3].point[1].y = rectMapCoordinates.maxy;
 
-#ifdef USE_PROJ
     if(layer->project)
       msProjectLine(&layer->map->projection, &layer->projection, &pInfo->pboundinglines[3]);
-#endif
   }
 
   return MS_SUCCESS;
@@ -660,10 +651,8 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
     searchrect.maxy = map->height-1;
   }
 
-#ifdef USE_PROJ
   if((map->projection.numargs > 0) && (layer->projection.numargs > 0))
     msProjectRect(&map->projection, &layer->projection, &searchrect); /* project the searchrect to source coords */
-#endif
 
  status =  msLayerOpen(layer);
  if(status != MS_SUCCESS)
@@ -702,8 +691,15 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
     msCopyShape(&shapegrid, &tmpshape);
     /* status = msDrawShape(map, layer, &tmpshape, image, -1); */
 
-    if(layer->project)
-      msProjectShape(&layer->projection, &map->projection, &shapegrid);
+    if(layer->project) {
+      if( layer->reprojectorLayerToMap == NULL )
+      {
+        layer->reprojectorLayerToMap = msProjectCreateReprojector(
+            &layer->projection, &map->projection);
+      }
+      if( layer->reprojectorLayerToMap )
+        msProjectShapeEx(layer->reprojectorLayerToMap, &shapegrid);
+    }
 
     msClipPolylineRect(&shapegrid, cliprect);
 
@@ -1067,14 +1063,19 @@ static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticule
 
   ptPoint = pShape->line->point[0];
 
-#ifdef USE_PROJ
   if(pLayer->project)
   {
-    msProjectShape( &pLayer->projection, &pLayer->map->projection, pShape );
+    if( pLayer->reprojectorLayerToMap == NULL )
+    {
+        pLayer->reprojectorLayerToMap = msProjectCreateReprojector(
+            &pLayer->projection, &pLayer->map->projection);
+    }
+    if( pLayer->reprojectorLayerToMap )
+        msProjectShapeEx(pLayer->reprojectorLayerToMap, pShape );
 
     /* Poor man detection of reprojection failure */
-    if( pj_is_latlong(pLayer->projection.proj) != 
-        pj_is_latlong(pLayer->map->projection.proj) )
+    if( msProjIsGeographicCRS(&(pLayer->projection)) != 
+        msProjIsGeographicCRS(&(pLayer->map->projection)) )
     {
         if( ptPoint.x == pShape->line->point[0].x &&
             ptPoint.y == pShape->line->point[0].y )
@@ -1083,7 +1084,6 @@ static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticule
         }
     }
   }
-#endif
 
   if(pLayer->transform) {
     msTransformShapeToPixelRound(pShape, pLayer->map->extent, pLayer->map->cellsize);
@@ -1122,13 +1122,12 @@ static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticule
   if(pLayer->transform)
     msTransformPixelToShape( pShape, pLayer->map->extent, pLayer->map->cellsize );
 
-#ifdef USE_PROJ
   if(pLayer->project)
   {
     /* Clamp coordinates into the validity area of the projection, in the */
     /* particular case of EPSG:3857 (WebMercator) to longlat reprojection */
     if( strstr(pLayer->map->projection.args[0], "epsg:3857") &&
-        pj_is_latlong(pLayer->projection.proj) )
+        msProjIsGeographicCRS(&(pLayer->projection)) )
     {
         if( !pLayer->map->projection.gt.need_geotransform &&
             ePosition == posLeft && pShape->line->point[0].x < -20037508)
@@ -1191,9 +1190,14 @@ static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticule
         }
     }
 
-    msProjectShape( &pLayer->map->projection, &pLayer->projection, pShape );
+    if( pLayer->reprojectorMapToLayer == NULL )
+    {
+        pLayer->reprojectorMapToLayer = msProjectCreateReprojector(
+            &pLayer->map->projection, &pLayer->projection);
+    }
+    if( pLayer->reprojectorMapToLayer )
+        msProjectShapeEx(pLayer->reprojectorMapToLayer, pShape );
   }
-#endif
 
   switch( ePosition ) {
     case posBottom:
