@@ -37,6 +37,7 @@
 */
 
 #include <stdarg.h>
+#include "maphash.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -55,7 +56,7 @@ extern "C" {
   int MS_DLL_EXPORT msIO_fprintf( FILE *stream, const char *format, ... ) MS_PRINT_FUNC_FORMAT(2,3);
   int MS_DLL_EXPORT msIO_fwrite( const void *ptr, size_t size, size_t nmemb, FILE *stream );
   int MS_DLL_EXPORT msIO_fread( void *ptr, size_t size, size_t nmemb, FILE *stream );
-  int MS_DLL_EXPORT msIO_vfprintf( FILE *fp, const char *format, va_list ap );
+  int MS_DLL_EXPORT msIO_vfprintf( FILE *fp, const char *format, va_list ap ) MS_PRINT_FUNC_FORMAT(2,0);
 
   /*
   ** Definitions for the callback function and the details of the IO
@@ -105,7 +106,8 @@ extern "C" {
   void MS_DLL_EXPORT msIO_Cleanup(void);
   char MS_DLL_EXPORT *msIO_stripStdoutBufferContentType(void);
   void MS_DLL_EXPORT msIO_stripStdoutBufferContentHeaders(void);
-  
+  hashTableObj MS_DLL_EXPORT *msIO_getAndStripStdoutBufferMimeHeaders(void);
+
   msIOContext *msIO_pushStdoutToBufferAndGetOldContext(void);
   void msIO_restoreOldStdoutContext(msIOContext *context_to_restore);
 
