@@ -34,6 +34,7 @@
 #include "mapserver.h"
 #include "mapthread.h"
 
+#include "cpl_vsi.h"
 
 #include <ctype.h>
 #include <string.h>
@@ -663,7 +664,7 @@ char *msStripPath(char *fn)
 /*
 ** Returns the *path* portion of the filename fn. Memory is allocated using malloc.
 */
-char *msGetPath(char *fn)
+char *msGetPath(const char *fn)
 {
   char *str;
   int i, length;
@@ -760,17 +761,17 @@ char *msBuildPath3(char *pszReturnPath, const char *abs_path, const char *path1,
 char *msTryBuildPath(char *szReturnPath, const char *abs_path, const char *path)
 
 {
-  FILE  *fp;
+  VSILFILE  *fp;
 
   if( msBuildPath( szReturnPath, abs_path, path ) == NULL )
     return NULL;
 
-  fp = fopen( szReturnPath, "r" );
+  fp = VSIFOpenL( szReturnPath, "r" );
   if( fp == NULL ) {
     strlcpy( szReturnPath, path, MS_MAXPATHLEN);
     return NULL;
   } else
-    fclose( fp );
+    VSIFCloseL( fp );
 
   return szReturnPath;
 }
@@ -785,17 +786,17 @@ char *msTryBuildPath(char *szReturnPath, const char *abs_path, const char *path)
 char *msTryBuildPath3(char *szReturnPath, const char *abs_path, const char *path1, const char *path2)
 
 {
-  FILE  *fp;
+  VSILFILE  *fp;
 
   if( msBuildPath3( szReturnPath, abs_path, path1, path2 ) == NULL )
     return NULL;
 
-  fp = fopen( szReturnPath, "r" );
+  fp = VSIFOpenL( szReturnPath, "r" );
   if( fp == NULL ) {
     strlcpy( szReturnPath, path2, MS_MAXPATHLEN);
     return NULL;
   } else
-    fclose( fp );
+    VSIFCloseL( fp );
 
   return szReturnPath;
 }
@@ -1501,9 +1502,9 @@ char *msCommifyString(char *str)
 
 
 /* ------------------------------------------------------------------------------- */
-/*       Replace all occurances of old with new in str.                            */
+/*       Replace all occurrences of old with new in str.                           */
 /*       It is assumed that str was dynamically created using malloc.              */
-/*       Same function as msReplaceSubstring but this is case incensitive                        */
+/*       Same function as msReplaceSubstring but this is case insensitive          */
 /* ------------------------------------------------------------------------------- */
 char *msCaseReplaceSubstring(char *str, const char *old, const char *new)
 {
@@ -2232,3 +2233,82 @@ int msLayerEncodeShapeAttributes( layerObj *layer, shapeObj *shape) {
   return MS_FAILURE;
 #endif
 }
+
+/************************************************************************/
+/*                             msStringBuffer                           */
+/************************************************************************/
+
+struct msStringBuffer
+{
+    size_t alloc_size;
+    size_t length;
+    char  *str;
+};
+
+/************************************************************************/
+/*                         msStringBufferAlloc()                        */
+/************************************************************************/
+
+msStringBuffer* msStringBufferAlloc(void)
+{
+    return (msStringBuffer*)msSmallCalloc(sizeof(msStringBuffer), 1);
+}
+
+/************************************************************************/
+/*                         msStringBufferFree()                         */
+/************************************************************************/
+
+void msStringBufferFree(msStringBuffer* sb)
+{
+    if( sb )
+        msFree(sb->str);
+    msFree(sb);
+}
+
+/************************************************************************/
+/*                       msStringBufferGetString()                      */
+/************************************************************************/
+
+const char* msStringBufferGetString(msStringBuffer* sb)
+{
+    return sb->str;
+}
+
+/************************************************************************/
+/*                   msStringBufferReleaseStringAndFree()               */
+/************************************************************************/
+
+char* msStringBufferReleaseStringAndFree(msStringBuffer* sb)
+{
+    char* str = sb->str;
+    sb->str = NULL;
+    sb->alloc_size = 0;
+    sb->length = 0;
+    msStringBufferFree(sb);
+    return str;
+}
+
+/************************************************************************/
+/*                        msStringBufferAppend()                        */
+/************************************************************************/
+
+int msStringBufferAppend(msStringBuffer* sb, const char* pszAppendedString)
+{
+    size_t nAppendLen = strlen(pszAppendedString);
+    if( sb->length + nAppendLen >= sb->alloc_size )
+    {
+        size_t newAllocSize1 = sb->alloc_size + sb->alloc_size / 3;
+        size_t newAllocSize2 = sb->length + nAppendLen + 1;
+        size_t newAllocSize = MAX(newAllocSize1, newAllocSize2);
+        void* newStr = realloc(sb->str, newAllocSize);
+        if( newStr == NULL ) {
+            msSetError(MS_MEMERR, "Not enough memory", "msStringBufferAppend()");
+            return MS_FAILURE;
+        }
+        sb->alloc_size = newAllocSize;
+        sb->str = (char*) newStr;
+    }
+    memcpy(sb->str + sb->length, pszAppendedString, nAppendLen + 1);
+    sb->length += nAppendLen;
+    return MS_SUCCESS;
+}
